# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import json
import logging
from pkg_resources import iter_entry_points

from http import HTTPStatus

from flask import Response, jsonify, make_response, request, current_app as app
from flask.blueprints import Blueprint
from marshmallow import ValidationError
from werkzeug.utils import import_string

from amundsen_application.models.preview_data import PreviewDataSchema

LOGGER = logging.getLogger(__name__)
PREVIEW_CLIENT_CLASS = None
PREVIEW_CLIENT_INSTANCE = None

for entry_point in iter_entry_points(group='preview_client', name='table_preview_client_class'):
    preview_client_class = entry_point.load()
    if preview_client_class is not None:
        PREVIEW_CLIENT_CLASS = preview_client_class

preview_blueprint = Blueprint('preview', __name__, url_prefix='/api/preview/v0')


@preview_blueprint.route('/', methods=['POST'])
def get_table_preview() -> Response:
    global PREVIEW_CLIENT_INSTANCE
    global PREVIEW_CLIENT_CLASS
    try:
        if PREVIEW_CLIENT_INSTANCE is None:
            if PREVIEW_CLIENT_CLASS is not None:
                PREVIEW_CLIENT_INSTANCE = PREVIEW_CLIENT_CLASS()
                logging.warn('Setting preview_client via entry_point is DEPRECATED and '
                             'will be removed in a future version')
            elif (app.config['PREVIEW_CLIENT_ENABLED']
                    and app.config['PREVIEW_CLIENT'] is not None):
                PREVIEW_CLIENT_CLASS = import_string(app.config['PREVIEW_CLIENT'])
                PREVIEW_CLIENT_INSTANCE = PREVIEW_CLIENT_CLASS()
            else:
                payload = jsonify({'previewData': {}, 'msg': 'A client for the preview feature must be configured'})
                return make_response(payload, HTTPStatus.NOT_IMPLEMENTED)

        response = PREVIEW_CLIENT_INSTANCE.get_preview_data(params=request.get_json())
        status_code = response.status_code

        preview_data = json.loads(response.data).get('preview_data')
        if status_code == HTTPStatus.OK:
            # validate the returned table preview data
            try:
                data = PreviewDataSchema().load(preview_data)
                payload = jsonify({'previewData': data, 'msg': 'Success'})
            except ValidationError as err:
                logging.error('Preview data dump returned errors: ' + str(err.messages))
                raise Exception('The preview client did not return a valid PreviewData object')
        else:
            message = 'Encountered error: Preview client request failed with code ' + str(status_code)
            logging.error(message)
            # only necessary to pass the error text
            payload = jsonify({'previewData': {'error_text': preview_data.get('error_text', '')}, 'msg': message})
        return make_response(payload, status_code)
    except Exception as e:
        message = f'Encountered exception: {str(e)}'
        logging.exception(message)
        payload = jsonify({'previewData': {}, 'msg': message})
        return make_response(payload, HTTPStatus.INTERNAL_SERVER_ERROR)


@preview_blueprint.route('/feature_preview', methods=['POST'])
def get_feature_preview() -> Response:
    global PREVIEW_CLIENT_INSTANCE
    global PREVIEW_CLIENT_CLASS
    try:
        if PREVIEW_CLIENT_INSTANCE is None:
            if PREVIEW_CLIENT_CLASS is not None:
                PREVIEW_CLIENT_INSTANCE = PREVIEW_CLIENT_CLASS()
                logging.warn('Setting preview_client via entry_point is DEPRECATED and '
                             'will be removed in a future version')
            elif (app.config['PREVIEW_CLIENT_ENABLED']
                    and app.config['PREVIEW_CLIENT'] is not None):
                PREVIEW_CLIENT_CLASS = import_string(app.config['PREVIEW_CLIENT'])
                PREVIEW_CLIENT_INSTANCE = PREVIEW_CLIENT_CLASS()
            else:
                payload = jsonify({'previewData': {}, 'msg': 'A client for the preview feature must be configured'})
                return make_response(payload, HTTPStatus.NOT_IMPLEMENTED)

        response = PREVIEW_CLIENT_INSTANCE.get_feature_preview_data(params=request.get_json())
        status_code = response.status_code

        preview_data = json.loads(response.data).get('preview_data')
        if status_code == HTTPStatus.OK:
            # validate the returned feature preview data
            try:
                data = PreviewDataSchema().load(preview_data)
                payload = jsonify({'previewData': data, 'msg': 'Success'})
            except ValidationError as err:
                logging.error('Preview data dump returned errors: ' + str(err.messages))
                raise Exception('The preview client did not return a valid PreviewData object')
        else:
            message = 'Encountered error: Preview client request failed with code ' + str(status_code)
            logging.error(message)
            # only necessary to pass the error text
            payload = jsonify({'previewData': {'error_text': preview_data.get('error_text', '')}, 'msg': message})
        return make_response(payload, status_code)
    except Exception as e:
        message = f'Encountered exception: {str(e)}'
        logging.exception(message)
        payload = jsonify({'previewData': {}, 'msg': message})
        return make_response(payload, HTTPStatus.INTERNAL_SERVER_ERROR)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Tuple
import logging

from flask import Flask, render_template, make_response
import jinja2
import os


ENVIRONMENT = os.getenv('APPLICATION_ENV', 'development')
LOGGER = logging.getLogger(__name__)


def init_routes(app: Flask) -> None:
    frontend_base = app.config.get('FRONTEND_BASE')
    config_override_enabled = app.config.get('JS_CONFIG_OVERRIDE_ENABLED')

    app.add_url_rule('/healthcheck', 'healthcheck', healthcheck)
    app.add_url_rule('/opensearch.xml', 'opensearch.xml', opensearch, defaults={'frontend_base': frontend_base})
    app.add_url_rule('/', 'index', index, defaults={'path': '',
                                                    'config_override_enabled': config_override_enabled,
                                                    'frontend_base': frontend_base})  # also functions as catch_all
    app.add_url_rule('/<path:path>', 'index', index,
                     defaults={'frontend_base': frontend_base,
                               'config_override_enabled': config_override_enabled})  # catch_all


def index(path: str, frontend_base: str, config_override_enabled: bool) -> Any:
    try:
        return render_template("index.html", env=ENVIRONMENT, frontend_base=frontend_base,
                               config_override_enabled=config_override_enabled)  # pragma: no cover
    except jinja2.exceptions.TemplateNotFound as e:
        LOGGER.error("index.html template not found, have you built the front-end JS (npm run build in static/?")
        raise e


def healthcheck() -> Tuple[str, int]:
    return '', 200  # pragma: no cover


def opensearch(frontend_base: str) -> Any:
    try:
        template = render_template("opensearch.xml", frontend_base=frontend_base)
        response = make_response(template)
        response.headers['Content-Type'] = 'application/xml'
        return response
    except jinja2.exceptions.TemplateNotFound as e:
        LOGGER.error("opensearch.xml template not found, have you built the front-end JS (npm run build in static/?")
        raise e



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import os
import warnings

from flask import Flask

warnings.simplefilter('always', DeprecationWarning)


# Deprecation Warnings
def process_deprecations(app: Flask) -> None:
    if os.getenv('APP_WRAPPER') or os.getenv('APP_WRAPPER_CLASS'):
        warnings.warn("'APP_WRAPPER' and 'APP_WRAPPER_CLASS' variables are deprecated since version (3.9.0), "
                      "and will be removed in version 4. "
                      "Please use 'FLASK_APP_MODULE_NAME' and 'FLASK_APP_CLASS_NAME' instead",
                      DeprecationWarning)

    if os.getenv('APP_WRAPPER_ARGS'):
        warnings.warn("'APP_WRAPPER_ARGS' variable is deprecated since version (3.9.0), "
                      "and will be removed in version 4. "
                      "Please use 'FLASK_APP_KWARGS_DICT' instead", DeprecationWarning)

    if app.config.get("POPULAR_TABLE_COUNT", None) is not None:
        app.config["POPULAR_RESOURCES_COUNT"] = app.config["POPULAR_TABLE_COUNT"]
        warnings.warn("'POPULAR_TABLE_COUNT' variable is deprecated since version (3.9.0), "
                      "and will be removed in version 4. "
                      "Please use 'POPULAR_RESOURCES_COUNT' instead", DeprecationWarning)

    if app.config.get("POPULAR_TABLE_PERSONALIZATION", None) is not None:
        app.config["POPULAR_RESOURCES_PERSONALIZATION"] = app.config["POPULAR_TABLE_PERSONALIZATION"]
        warnings.warn("'POPULAR_TABLE_PERSONALIZATION' variable is deprecated since version (3.9.0), "
                      "and will be removed in version 4. "
                      "Please use 'POPULAR_RESOURCES_PERSONALIZATION' instead", DeprecationWarning)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from http import HTTPStatus
from jira import JIRA, JIRAError, Issue, User as JiraUser
from typing import Any, List

from flask import current_app as app

from amundsen_application.api.metadata.v0 import USER_ENDPOINT
from amundsen_application.api.utils.request_utils import request_metadata
from amundsen_application.base.base_issue_tracker_client import BaseIssueTrackerClient
from amundsen_application.proxy.issue_tracker_clients.issue_exceptions import IssueConfigurationException
from amundsen_application.models.data_issue import DataIssue, Priority
from amundsen_application.models.issue_results import IssueResults
from amundsen_application.models.user import load_user
from amundsen_common.models.user import User

import urllib.parse
import logging

SEARCH_STUB_ALL_ISSUES = ('text ~ "\\"Table Key: {table_key} [PLEASE DO NOT REMOVE]\\"" '
                          'and (resolution = unresolved or (resolution != unresolved and updated > -30d)) '
                          'order by resolution DESC, priority DESC, createdDate DESC')
SEARCH_STUB_OPEN_ISSUES = ('text ~ "\\"Table Key: {table_key} [PLEASE DO NOT REMOVE]\\"" '
                           'and resolution = unresolved '
                           'order by priority DESC, createdDate DESC')
SEARCH_STUB_CLOSED_ISSUES = ('text ~ "\\"Table Key: {table_key} [PLEASE DO NOT REMOVE]\\"" '
                             'and resolution != unresolved '
                             'order by priority DESC, createdDate DESC')
# this is provided by jira as the type of a bug
ISSUE_TYPE_ID = 1
ISSUE_TYPE_NAME = 'Bug'


class JiraClient(BaseIssueTrackerClient):

    def __init__(self, issue_labels: List[str],
                 issue_tracker_url: str,
                 issue_tracker_user: str,
                 issue_tracker_password: str,
                 issue_tracker_project_id: int,
                 issue_tracker_max_results: int) -> None:
        self.issue_labels = issue_labels
        self.jira_url = issue_tracker_url
        self.jira_user = issue_tracker_user
        self.jira_password = issue_tracker_password
        self.jira_project_id = issue_tracker_project_id
        self.jira_max_results = issue_tracker_max_results
        self._validate_jira_configuration()
        self.jira_client = self.get_client()

    def get_client(self) -> JIRA:
        """
        Get the Jira client properly formatted prepared for hitting JIRA
        :return: A Jira client.
        """
        return JIRA(
            server=self.jira_url,
            basic_auth=(self.jira_user, self.jira_password)
        )

    def get_issues(self, table_uri: str) -> IssueResults:
        """
        Runs a query against a given Jira project for tickets matching the key
        Returns open issues sorted by most recently created.
        :param table_uri: Table Uri ie databasetype://database/table
        :return: Metadata of matching issues
        """
        try:
            issues = self.jira_client.search_issues(SEARCH_STUB_ALL_ISSUES.format(
                table_key=table_uri),
                maxResults=self.jira_max_results)

            # Call search_issues for only 1 open/closed issue just to get the total values from the response. The
            # total count from all issues may not be accurate if older closed issues are excluded from the response
            open_issues = self.jira_client.search_issues(SEARCH_STUB_OPEN_ISSUES.format(
                table_key=table_uri),
                maxResults=1)
            closed_issues = self.jira_client.search_issues(SEARCH_STUB_CLOSED_ISSUES.format(
                table_key=table_uri),
                maxResults=1)

            returned_issues = self._sort_issues(issues)
            return IssueResults(issues=returned_issues,
                                total=open_issues.total + closed_issues.total,
                                all_issues_url=self._generate_issues_url(SEARCH_STUB_ALL_ISSUES,
                                                                         table_uri,
                                                                         open_issues.total + closed_issues.total),
                                open_issues_url=self._generate_issues_url(SEARCH_STUB_OPEN_ISSUES,
                                                                          table_uri,
                                                                          open_issues.total),
                                closed_issues_url=self._generate_issues_url(SEARCH_STUB_CLOSED_ISSUES,
                                                                            table_uri,
                                                                            closed_issues.total),
                                open_count=open_issues.total)
        except JIRAError as e:
            logging.exception(str(e))
            raise e

    def create_issue(self,
                     table_uri: str,
                     title: str,
                     description: str,
                     priority_level: str,
                     table_url: str,
                     **kwargs: Any) -> DataIssue:
        """
        Creates an issue in Jira
        :param description: Description of the Jira issue
        :param priority_level: Priority level for the ticket
        :param table_uri: Table Uri ie databasetype://database/table
        :param title: Title of the Jira ticket
        :param table_url: Link to access the table
        :param owner_ids: List of table owners user ids
        :param frequent_user_ids: List of table frequent users user ids
        :param project_key: Jira project key to specify where the ticket should be created
        :return: Metadata about the newly created issue
        """
        try:
            if app.config['AUTH_USER_METHOD']:
                user_email = app.config['AUTH_USER_METHOD'](app).email
                # We currently cannot use the email directly because of the following issue:
                # https://community.atlassian.com/t5/Answers-Developer-Questions/JIRA-Rest-API-find-JIRA-user-based-on-user-s-email-address/qaq-p/532715
                jira_id = user_email.split('@')[0]
            else:
                raise Exception('AUTH_USER_METHOD must be configured to set the JIRA issue reporter')

            reporter = {'name': jira_id}

            # Detected by the jira client based on API version & deployment.
            if self.jira_client.deploymentType == 'Cloud':
                try:
                    user = self.jira_client._fetch_pages(JiraUser, None, "user/search", 0, 1, {'query': user_email})[0]
                    reporter = {'accountId': user.accountId}
                except IndexError:
                    raise Exception('Could not find the reporting user in our Jira installation.')

            issue_type_id = ISSUE_TYPE_ID
            if app.config['ISSUE_TRACKER_ISSUE_TYPE_ID']:
                issue_type_id = app.config['ISSUE_TRACKER_ISSUE_TYPE_ID']

            project_key = kwargs.get('project_key', None)
            proj_key = 'key' if project_key else 'id'
            proj_value = project_key if project_key else self.jira_project_id

            reporting_user = self._get_users_from_ids([user_email])
            owners = self._get_users_from_ids(kwargs.get('owner_ids', []))
            frequent_users = self._get_users_from_ids(kwargs.get('frequent_user_ids', []))

            reporting_user_str = self._generate_reporting_user_str(reporting_user)
            owners_description_str = self._generate_owners_description_str(owners)
            frequent_users_description_str = self._generate_frequent_users_description_str(frequent_users)
            all_users_description_str = self._generate_all_table_users_description_str(owners_description_str,
                                                                                       frequent_users_description_str)

            issue = self.jira_client.create_issue(fields=dict(project={
                proj_key: proj_value
            }, issuetype={
                'id': issue_type_id,
                'name': ISSUE_TYPE_NAME,
            }, labels=self.issue_labels,
                summary=title,
                description=(f'{description} '
                             f'\n *Reported By:* {reporting_user_str if reporting_user_str else user_email} '
                             f'\n *Table Key:* {table_uri} [PLEASE DO NOT REMOVE] '
                             f'\n *Table URL:* {table_url} '
                             f'{all_users_description_str}'),
                priority={
                    'name': Priority.get_jira_severity_from_level(priority_level)
            }, reporter=reporter))

            self._add_watchers_to_issue(issue_key=issue.key, users=owners + frequent_users)

            return self._get_issue_properties(issue=issue)
        except JIRAError as e:
            logging.exception(str(e))
            raise e

    def _validate_jira_configuration(self) -> None:
        """
        Validates that all properties for jira configuration are set. Returns a list of missing properties
        to return if they are missing
        :return: String representing missing Jira properties, or an empty string.
        """
        missing_fields = []
        if not self.jira_url:
            missing_fields.append('ISSUE_TRACKER_URL')
        if not self.jira_user:
            missing_fields.append('ISSUE_TRACKER_USER')
        if not self.jira_password:
            missing_fields.append('ISSUE_TRACKER_PASSWORD')
        if not self.jira_project_id:
            missing_fields.append('ISSUE_TRACKER_PROJECT_ID')
        if not self.jira_max_results:
            missing_fields.append('ISSUE_TRACKER_MAX_RESULTS')

        if missing_fields:
            raise IssueConfigurationException(
                f'The following config settings must be set for Jira: {", ".join(missing_fields)} ')

    @staticmethod
    def _get_issue_properties(issue: Issue) -> DataIssue:
        """
        Maps the jira issue object to properties we want in the UI
        :param issue: Jira issue to map
        :return: JiraIssue
        """
        return DataIssue(issue_key=issue.key,
                         title=issue.fields.summary,
                         url=issue.permalink(),
                         status=issue.fields.status.name,
                         priority=Priority.from_jira_severity(issue.fields.priority.name))

    def _generate_issues_url(self, search_stub: str, table_uri: str, issueCount: int) -> str:
        """
        Way to get list of jira tickets
        SDK doesn't return a query
        :param search_stub: search stub for type of query to build
        :param table_uri: table uri from the ui
        :param issueCount: number of jira issues associated to the search
        :return: url to a list of issues in jira
        """
        if issueCount == 0:
            return ''
        search_query = urllib.parse.quote(search_stub.format(table_key=table_uri))
        return f'{self.jira_url}/issues/?jql={search_query}'

    def _sort_issues(self, issues: List[Issue]) -> List[DataIssue]:
        """
        Sorts issues by resolution, first by unresolved and then by resolved. Also maps the issues to
        the object used by the front end. Doesn't include closed issues that are older than 30 days.
        :param issues: Issues returned from the JIRA API
        :return: List of data issues
        """
        open = []
        closed = []
        for issue in issues:
            data_issue = self._get_issue_properties(issue)
            if not issue.fields.resolution:
                open.append(data_issue)
            else:
                closed.append(data_issue)
        return open + closed

    @staticmethod
    def _get_users_from_ids(user_ids: List[str]) -> User:
        """
        Calls get_user metadata API with a user id to retrieve user details.
        :param user_ids: List of strings representing user ids
        :return: List of User objects
        """
        users = []
        for user_id in user_ids:
            url = '{0}{1}/{2}'.format(app.config['METADATASERVICE_BASE'], USER_ENDPOINT, user_id)
            response = request_metadata(url=url)
            if response.status_code == HTTPStatus.OK:
                user = load_user(response.json())
                if user:
                    users.append(user)
        return users

    def _generate_reporting_user_str(self, reporting_user: List[User]) -> str:
        """
        :param reporting_user: List containing a user representing the reporter of the issue
        or an empty list if the reporter's information could not be retrieved
        :return: String of reporting user's information to display in the description
        """
        if not reporting_user:
            return ''
        user = reporting_user[0]
        if user.is_active and user.profile_url:
            return (f'[{user.full_name if user.full_name else user.email}'
                    f'|{user.profile_url}]')
        else:
            return user.email

    def _generate_owners_description_str(self, owners: List[User]) -> str:
        """
        Build a list of table owner information to add to the description of the ticket
        :param owners: List of users representing owners of the table
        :return: String of owners to append in the description
        """
        owners_description_str = '\n Table Owners:' if owners else ''
        user_details_list = []
        inactive_user_details_list = []
        for user in owners:
            if user.is_active and user.profile_url:
                user_details_list.append((f'[{user.full_name if user.full_name else user.email}'
                                          f'|{user.profile_url}] '))
                continue
            else:
                inactive_user_details = f'{user.full_name if user.full_name else user.email}'

            # Append relevant alumni and manager information if the user is a person and inactive
            if not user.is_active and user.full_name:
                inactive_user_details += ' (Alumni) '
                if user.manager_fullname:
                    inactive_user_details += f'\u2022 Manager: {user.manager_fullname} '
            inactive_user_details_list.append(inactive_user_details)
        return '\n '.join(filter(None, [owners_description_str,
                                        '\n '.join(user_details_list),
                                        '\n '.join(inactive_user_details_list)]))

    def _generate_frequent_users_description_str(self, frequent_users: List[User]) -> str:
        """
        Build a list of table frequent user information to add to the description of the ticket; this list will leave
        out inactive frequent users
        :param frequent_users: List of users representing frequent users of the table
        :return: String of frequent users to append in the description
        """
        frequent_users_description_str = '\n Frequent Users: ' if frequent_users else ''
        user_details_list = []
        for user in frequent_users:
            if user.is_active and user.profile_url:
                user_details_list.append((f'[{user.full_name if user.full_name else user.email}'
                                          f'|{user.profile_url}]'))
        return frequent_users_description_str + ', '.join(user_details_list) if user_details_list else ''

    def _generate_all_table_users_description_str(self, owners_str: str, frequent_users_str: str) -> str:
        """
        Takes the generated owners and frequent users information and packages it up into one string for appending
        to the ticket description
        :param owners_str: Owner information
        :param frequent_users_str: Frequent user information
        :return: String including all table users (owners and frequent users) information to append to the description
        """
        table_users_description_title = ''
        if owners_str and frequent_users_str:
            table_users_description_title = '\n\n *Owners and Frequent Users (added as Watchers):* '
        elif owners_str:
            table_users_description_title = '\n\n *Owners (added as Watchers):* '
        elif frequent_users_str:
            table_users_description_title = '\n\n *Frequent Users (added as Watchers):* '
        return table_users_description_title + owners_str + frequent_users_str

    def _add_watchers_to_issue(self, issue_key: str, users: List[User]) -> None:
        """
        Given an issue key and a list of users, add those users as watchers to the issue if they are active
        :param issue_key: key representing an issue
        :param users: list of users to add as watchers to the issue
        """
        for user in users:
            if user.is_active:
                try:
                    # Detected by the jira client based on API version & deployment.
                    if self.jira_client.deploymentType == 'Cloud':
                        jira_user = self.jira_client._fetch_pages(JiraUser, None, "user/search", 0, 1,
                                                                  {'query': user.email})[0]
                        self.jira_client.add_watcher(issue=issue_key, watcher=jira_user.accountId)
                    else:
                        self.jira_client.add_watcher(issue=issue_key, watcher=user.email.split("@")[0])
                except (JIRAError, IndexError):
                    logging.warning('Could not add user {user_email} as a watcher on the issue.'
                                    .format(user_email=user.email))



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import io
import logging
from http import HTTPStatus

from flask import send_file, jsonify, make_response, Response, current_app as app
from flask.blueprints import Blueprint

from amundsen_application.api.preview.dashboard.dashboard_preview.preview_factory_method import \
    DefaultPreviewMethodFactory, BasePreviewMethodFactory

LOGGER = logging.getLogger(__name__)
PREVIEW_FACTORY: BasePreviewMethodFactory = None  # type: ignore

dashboard_preview_blueprint = Blueprint('dashboard_preview', __name__, url_prefix='/api/dashboard_preview/v0')


def initialize_preview_factory_class() -> None:
    """
    Instantiates Preview factory class and assign it to PREVIEW_FACTORY
    :return: None
    """
    global PREVIEW_FACTORY

    PREVIEW_FACTORY = app.config['DASHBOARD_PREVIEW_FACTORY']
    if not PREVIEW_FACTORY:
        PREVIEW_FACTORY = DefaultPreviewMethodFactory()

    LOGGER.info('Using {} for Dashboard'.format(PREVIEW_FACTORY))


@dashboard_preview_blueprint.route('/dashboard/<path:uri>/preview.jpg', methods=['GET'])
def get_preview_image(uri: str) -> Response:
    """
    Provides preview image of Dashboard which can be cached for a day (by default).
    :return:
    """

    if not PREVIEW_FACTORY:
        LOGGER.info('Initializing Dashboard PREVIEW_FACTORY')
        initialize_preview_factory_class()

    preview_client = PREVIEW_FACTORY.get_instance(uri=uri)
    try:
        return send_file(io.BytesIO(preview_client.get_preview_image(uri=uri)),
                         mimetype='image/jpeg',
                         cache_timeout=app.config['DASHBOARD_PREVIEW_IMAGE_CACHE_MAX_AGE_SECONDS'])
    except FileNotFoundError as fne:
        LOGGER.exception('FileNotFoundError on get_preview_image')
        return make_response(jsonify({'msg': fne.args[0]}), HTTPStatus.NOT_FOUND)
    except PermissionError as pe:
        LOGGER.exception('PermissionError on get_preview_image')
        return make_response(jsonify({'msg': pe.args[0]}), HTTPStatus.UNAUTHORIZED)
    except Exception as e:
        LOGGER.exception('Unexpected failure on get_preview_image')
        return make_response(jsonify({'msg': 'Encountered exception: ' + str(e)}), HTTPStatus.INTERNAL_SERVER_ERROR)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from typing import Optional

import attr
from marshmallow3_annotations.ext.attrs import AttrsSchema


@attr.s(auto_attribs=True, kw_only=True)
class TableQualityChecksSummary:
    num_checks_success: int = attr.ib()
    num_checks_failed: int = attr.ib()
    num_checks_total: int = attr.ib()
    external_url: str = attr.ib()
    last_run_timestamp: Optional[int] = attr.ib()


class TableQualityChecksSchema(AttrsSchema):
    class Meta:
        target = TableQualityChecksSummary
        register_as_schema = True



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import json
import os
from typing import Dict

import boto3
from amundsen_application.base.base_s3_preview_client import \
    BaseS3PreviewClient
from amundsen_application.models.preview_data import ColumnItem, PreviewData


class S3JSONPreviewClient(BaseS3PreviewClient):
    """
    S3JSONPreviewClient is an S3 Preview Client that:
    1. Gets JSON files from S3 that are stored in a bucket with keys preview_data/{schema}/{table}.json
    2. Converts the JSON values to PreviewData model
    3. Returns the serialized model

    In order for this preview client to work you must:
    - Have S3 files stored in a bucket with keys 'preview_data/{schema}/{table}.json'
    - Files are formatted as list of rows as map with key being the column name and value being column value
      Ex:
    [
      {
        'col1': 1,
        'col2': '2'
      },
      {
        'col1': 3,
        'col2': '4'
      }
      ...
    ]
    - Nested field are not supported. We suggest flattening your nested fields.
      Ex:
        [
          {
            'col1': {
              'col2: 1
          }
        ]
      should be:
        [
          {
            'col1.col2': 1
          }
        ]
    - Run your frontend service with an IAM Profile that has s3:GetObject permissions on the 'preview_data/' prefix
    """

    def __init__(self) -> None:
        self.s3 = boto3.client("s3")
        bucket = os.getenv("PREVIEW_CLIENT_S3_BUCKET")
        if bucket == "":
            raise Exception("When using the S3CSVPreviewClient you must set the PREVIEW_CLIENT_S3_BUCKET environment "
                            "variable to point to where your preview_data CSVs are stored.")
        self.s3_bucket = bucket

    def get_s3_preview_data(self, *, params: Dict) -> PreviewData:
        schema = params.get("schema")
        table = params.get("tableName")

        try:
            obj = self.s3.get_object(Bucket=self.s3_bucket, Key=f"preview_data/{schema}/{table}.json")
        except Exception as e:
            raise Exception(f"Error getting object from s3. preview_data/{schema}/{table}.json"
                            f"Caused by: {e}")

        data = json.loads(obj['Body'].read().decode('utf-8'))
        columns = [ColumnItem(col_name, '') for col_name in data[0]]  # TODO: figure out how to do Type. Is it needed?
        return PreviewData(columns=columns, data=data)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from flask import current_app as app
from flask import jsonify, make_response, Response
from flask_restful import Resource, reqparse
from http import HTTPStatus
import logging

from amundsen_application.base.base_issue_tracker_client import BaseIssueTrackerClient
from amundsen_application.proxy.issue_tracker_clients import get_issue_tracker_client
from amundsen_application.proxy.issue_tracker_clients.issue_exceptions import IssueConfigurationException

LOGGER = logging.getLogger(__name__)


class IssuesAPI(Resource):
    def __init__(self) -> None:
        self.reqparse = reqparse.RequestParser()
        self.client: BaseIssueTrackerClient

    def get(self) -> Response:
        """
        Given a table key, returns all tickets containing that key. Returns an empty array if none exist
        :return: List of tickets
        """
        try:
            if not app.config['ISSUE_TRACKER_CLIENT_ENABLED']:
                message = 'Issue tracking is not enabled. Request was accepted but no issue will be returned.'
                logging.exception(message)
                return make_response(jsonify({'msg': message}), HTTPStatus.ACCEPTED)

            self.client = get_issue_tracker_client()
            self.reqparse.add_argument('key', 'Request requires a key', location='args')
            args = self.reqparse.parse_args()
            response = self.client.get_issues(args['key'])
            return make_response(jsonify({'issues': response.serialize()}), HTTPStatus.OK)

        except IssueConfigurationException as e:
            message = 'Encountered exception: ' + str(e)
            logging.exception(message)
            return make_response(jsonify({'msg': message}), HTTPStatus.NOT_IMPLEMENTED)
        except Exception as e:
            message = 'Encountered exception: ' + str(e)
            logging.exception(message)
            return make_response(jsonify({'msg': message}), HTTPStatus.INTERNAL_SERVER_ERROR)


class IssueAPI(Resource):
    def __init__(self) -> None:
        self.reqparse = reqparse.RequestParser()
        self.client: BaseIssueTrackerClient
        super(IssueAPI, self).__init__()

    def post(self) -> Response:
        try:
            if not app.config['ISSUE_TRACKER_CLIENT_ENABLED']:
                message = 'Issue tracking is not enabled. Request was accepted but no issue will be created.'
                logging.exception(message)
                return make_response(jsonify({'msg': message}), HTTPStatus.ACCEPTED)
            self.client = get_issue_tracker_client()

            self.reqparse.add_argument('title', type=str, location='json')
            self.reqparse.add_argument('key', type=str, location='json')
            self.reqparse.add_argument('description', type=str, location='json')
            self.reqparse.add_argument('owner_ids', type=list, location='json')
            self.reqparse.add_argument('frequent_user_ids', type=list, location='json')
            self.reqparse.add_argument('priority_level', type=str, location='json')
            self.reqparse.add_argument('project_key', type=str, location='json')
            self.reqparse.add_argument('resource_path', type=str, location='json')
            args = self.reqparse.parse_args()
            response = self.client.create_issue(description=args['description'],
                                                owner_ids=args['owner_ids'],
                                                frequent_user_ids=args['frequent_user_ids'],
                                                priority_level=args['priority_level'],
                                                project_key=args['project_key'],
                                                table_uri=args['key'],
                                                title=args['title'],
                                                table_url=app.config['FRONTEND_BASE'] + args['resource_path']
                                                if args['resource_path'] else 'Not Found')
            return make_response(jsonify({'issue': response.serialize()}), HTTPStatus.OK)

        except IssueConfigurationException as e:
            message = 'Encountered exception: ' + str(e)
            logging.exception(message)
            return make_response(jsonify({'msg': message}), HTTPStatus.NOT_IMPLEMENTED)
        except Exception as e:
            message = 'Encountered exception: ' + str(e)
            logging.exception(message)
            return make_response(jsonify({'msg': message}), HTTPStatus.INTERNAL_SERVER_ERROR)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import ast
import importlib
import logging
import logging.config
import os
import sys

from flask import Blueprint, Flask
from flask_restful import Api

from amundsen_application.api import init_routes
from amundsen_application.api.announcements.v0 import announcements_blueprint
from amundsen_application.api.issue.issue import IssueAPI, IssuesAPI
from amundsen_application.api.log.v0 import log_blueprint
from amundsen_application.api.mail.v0 import mail_blueprint
from amundsen_application.api.metadata.v0 import metadata_blueprint
from amundsen_application.api.preview.dashboard.v0 import \
    dashboard_preview_blueprint
from amundsen_application.api.preview.v0 import preview_blueprint
from amundsen_application.api.quality.v0 import quality_blueprint
from amundsen_application.api.search.v1 import search_blueprint
from amundsen_application.api.v0 import blueprint
from amundsen_application.deprecations import process_deprecations

# For customized flask use below arguments to override.

FLASK_APP_MODULE_NAME = os.getenv('FLASK_APP_MODULE_NAME') or os.getenv('APP_WRAPPER')
FLASK_APP_CLASS_NAME = os.getenv('FLASK_APP_CLASS_NAME') or os.getenv('APP_WRAPPER_CLASS')
FLASK_APP_KWARGS_DICT_STR = os.getenv('FLASK_APP_KWARGS_DICT') or os.getenv('APP_WRAPPER_ARGS')

""" Support for importing a subclass of flask.Flask, via env variables """
if FLASK_APP_MODULE_NAME and FLASK_APP_CLASS_NAME:
    print('Using requested Flask module {module_name} and class {class_name}'
          .format(module_name=FLASK_APP_MODULE_NAME, class_name=FLASK_APP_CLASS_NAME), file=sys.stderr)
    moduleName = FLASK_APP_MODULE_NAME
    module = importlib.import_module(moduleName)
    moduleClass = FLASK_APP_CLASS_NAME
    app_wrapper_class = getattr(module, moduleClass)  # type: ignore
else:
    app_wrapper_class = Flask

PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))
STATIC_ROOT = os.getenv('STATIC_ROOT', 'static')
static_dir = os.path.join(PROJECT_ROOT, STATIC_ROOT)


def create_app(config_module_class: str = None, template_folder: str = None) -> Flask:
    """ Support for importing arguments for a subclass of flask.Flask """
    args = ast.literal_eval(FLASK_APP_KWARGS_DICT_STR) if FLASK_APP_KWARGS_DICT_STR else {}

    tmpl_dir = template_folder if template_folder else os.path.join(PROJECT_ROOT, static_dir, 'dist/templates')
    app = app_wrapper_class(__name__, static_folder=static_dir, template_folder=tmpl_dir, **args)

    # Support for importing a custom config class
    if not config_module_class:
        config_module_class = os.getenv('FRONTEND_SVC_CONFIG_MODULE_CLASS')

    app.config.from_object(config_module_class)

    if app.config.get('LOG_CONFIG_FILE'):
        logging.config.fileConfig(app.config['LOG_CONFIG_FILE'], disable_existing_loggers=False)
    else:
        logging.basicConfig(format=app.config['LOG_FORMAT'], datefmt=app.config.get('LOG_DATE_FORMAT'))
        logging.getLogger().setLevel(app.config['LOG_LEVEL'])

    logging.info('Created app with config name {}'.format(config_module_class))
    logging.info('Using metadata service at {}'.format(app.config.get('METADATASERVICE_BASE')))
    logging.info('Using search service at {}'.format(app.config.get('SEARCHSERVICE_BASE')))

    api_bp = Blueprint('api', __name__)
    api = Api(api_bp)

    api.add_resource(IssuesAPI,
                     '/api/issue/issues', endpoint='issues')
    api.add_resource(IssueAPI,
                     '/api/issue/issue', endpoint='issue')

    app.register_blueprint(blueprint)
    app.register_blueprint(announcements_blueprint)
    app.register_blueprint(log_blueprint)
    app.register_blueprint(mail_blueprint)
    app.register_blueprint(metadata_blueprint)
    app.register_blueprint(preview_blueprint)
    app.register_blueprint(quality_blueprint)
    app.register_blueprint(search_blueprint)
    app.register_blueprint(api_bp)
    app.register_blueprint(dashboard_preview_blueprint)
    init_routes(app)

    init_custom_routes = app.config.get('INIT_CUSTOM_ROUTES')
    if init_custom_routes:
        init_custom_routes(app)

    # handles the deprecation warnings
    # and process any config/environment variables accordingly
    process_deprecations(app)

    return app



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import abc
from enum import Enum
import logging
import requests as r
import time

from flask import Response as FlaskResponse, make_response, jsonify
from http import HTTPStatus
from typing import Any, Dict, Optional, Tuple

from amundsen_application.base.base_preview_client import BasePreviewClient
from amundsen_application.models.preview_data import ColumnItem, PreviewData, PreviewDataSchema


LOGGER = logging.getLogger(__name__)


REDASH_SUBMIT_QUERY_ENDPOINT = '{redash_host}/api/queries/{query_id}/results'
REDASH_TRACK_JOB_ENDPOINT = '{redash_host}/api/jobs/{job_id}'
REDASH_QUERY_RESULTS_ENDPOINT = '{redash_host}/api/query_results/{query_result_id}'


class RedashApiKeyNotProvidedException(Exception):
    pass


class RedashQueryCouldNotCompleteException(Exception):
    pass


class RedashQueryTemplateDoesNotExistForResource(Exception):
    pass


class RedashApiResponse(Enum):
    PENDING = 1  # (waiting to be executed)
    STARTED = 2  # (executing)
    SUCCESS = 3
    FAILURE = 4
    CANCELLED = 5


class BaseRedashPreviewClient(BasePreviewClient):
    """
    Generic client for using Redash as a preview client backend.

    Redash does not allow arbitrary queries to be submitted but it does allow
    the creation of templated queries that can be saved and referenced. Amundsen
    uses these templated queries to pass in arguments such as the schema name
    and table name in order to dynamically build a query on the fly.

    The suggested format of the query template is:

        select {{ SELECT_FIELDS }}
        from {{ SCHEMA_NAME }}.{{ TABLE_NAME }}
        {{ WHERE_CLAUSE }}
        limit {{ RCD_LIMIT }}

    You will need to use the params (e.g. database, cluster, schema and table names)
    to idenfiy the specific query ID in Redash to use. This is done via the
    `get_redash_query_id` method.

    The template values in the Redash query will be filled by the `build_redash_query_params`
    function.
    """

    def __init__(self, redash_host: str, user_api_key: Optional[str] = None) -> None:
        self.redash_host = redash_host
        self.user_api_key: Optional[str] = user_api_key
        self.headers: Optional[Dict] = None
        self.default_query_limit = 50
        self.max_redash_cache_age = 86400  # One day

    @abc.abstractmethod
    def get_redash_query_id(self, params: Dict) -> Optional[int]:
        """
        Retrieves the query template that should be executed for the given
        source / database / schema / table combination.

        Redash Connections are generally unique to the source and database.
        For example, Snowflake account that has two databases would require two
        separate connections in Redash. This would require at least one query
        template per connection.

        The query ID can be found in the URL of the query when using the Redash GUI.

        :param params: A dictionary of input parameters containing the database,
            cluster, schema and tableName
        :returns: the ID for the query in Redash. Can be None if one does not exist.
        """
        pass  # pragma: no cover

    def _build_headers(self, params: Dict) -> None:
        """
        Generates the headers to use for the API invocation. Attemps to use a
        Query API key, if it exists, then falls back to a User API if no
        query API key is returned.

        Background on Redash API keys: https://redash.io/help/user-guide/integrations-and-api/api
        """
        api_key = self._get_query_api_key(params) or self.user_api_key
        if api_key is None:
            raise RedashApiKeyNotProvidedException('No API key provided')
        self.headers = {"Authorization": "Key {}".format(api_key)}

    def _get_query_api_key(self, params: Dict) -> Optional[str]:
        """
        This function can be overridden by sub classes to look up the specific
        API key to use for a given database / cluster / schema / table combination.
        """
        return None

    def get_select_fields(self, params: Dict) -> str:
        """
        Allows customization of the fields in the select clause. This can be used to
        return a subset of fields or to apply functions (e.g. to mask data) on a
        table by table basis. Defaults to `*` for all fields.

        This string should be valid SQL AND fit BETWEEN the brackets `SELECT {} FROM ...`

        :param params: A dictionary of input parameters containing the database,
            cluster, schema and tableName
        :returns: a string corresponding to fields to select in the query
        """
        return '*'

    def get_where_clause(self, params: Dict) -> str:
        """
        Allows customization of the 'WHERE' clause to be provided for each set of parameters
        by the client implementation. Defaults to an empty string.
        """
        return ''

    def build_redash_query_params(self, params: Dict) -> Dict:
        """
        Builds a dictionary of parameters that will be injected into the Redash query
        template. The keys in this dictionary MUST be a case-sensitive match to the
        template names in the Redash query and you MUST have the exact same parameters,
        no more, no less.

        Override this function to provide custom values.
        """
        return {
            'parameters': {
                'SELECT_FIELDS': self.get_select_fields(params),
                'SCHEMA_NAME': params.get('schema'),
                'TABLE_NAME': params.get('tableName'),
                'WHERE_CLAUSE': self.get_where_clause(params),
                'RCD_LIMIT': str(self.default_query_limit)
            },
            'max_age': self.max_redash_cache_age
        }

    def _start_redash_query(self, query_id: int, query_params: Dict) -> Tuple[Any, bool]:
        """
        Starts a query in Redash. Returns a job ID that can be used to poll for
        the job status.

        :param query_id: The ID of the query in the Redash system. This can
            be retrieved by viewing the URL for your query template in the
            Redash GUI.
        :param query_params: A dictionary of parameters to inject into the
            corresponding query's template
        :return: A tuple of the response object and boolean. The response object
            changes based off of whether or not the result from Redash came from
            the cache.
            The boolean is True if the result came from the Redash cache, otherwise False.
        """
        url_inputs = {'redash_host': self.redash_host, 'query_id': query_id}
        query_url = REDASH_SUBMIT_QUERY_ENDPOINT.format(**url_inputs)

        resp = r.post(query_url, json=query_params, headers=self.headers)
        resp_json = resp.json()

        LOGGER.debug('Response from redash query: %s', resp_json)

        # When submitting a query, Redash can return 2 distinct payloads. One if the
        # query result has been cached by Redash and one if the query was submitted
        # to be executed. The 'job' object is returned if the query is not cached.
        if 'job' in resp_json:
            redash_cached = False
        else:
            redash_cached = True

        return resp_json, redash_cached

    def _wait_for_query_finish(self, job_id: str, max_wait: int = 60) -> str:
        """
        Waits for the query to finish and validates that a successful response is returned.

        :param job_id: the ID for the job executing the query
        :return: a query result ID tha can be used to fetch the results
        """
        url_inputs = {'redash_host': self.redash_host, 'job_id': job_id}
        query_url = REDASH_TRACK_JOB_ENDPOINT.format(**url_inputs)

        query_result_id: Optional[str] = None
        max_time = time.time() + max_wait

        while time.time() < max_time:
            resp = r.get(query_url, headers=self.headers)
            resp_json = resp.json()

            LOGGER.debug('Received response from Redash job %s: %s', job_id, resp_json)

            job_info = resp_json['job']
            job_status = RedashApiResponse(job_info['status'])

            if job_status == RedashApiResponse.SUCCESS:
                query_result_id = job_info['query_result_id']
                break

            elif job_status == RedashApiResponse.FAILURE:
                raise RedashQueryCouldNotCompleteException(job_info['error'])
            time.sleep(.5)

        if query_result_id is None:
            raise RedashQueryCouldNotCompleteException('Query execution took too long')

        return query_result_id

    def _get_query_results(self, query_result_id: str) -> Dict:
        """
        Retrieves query results from a successful query run

        :param query_result_id: ID returned by Redash after a successful query execution
        :return: A Redash response dictionary
        """
        url_inputs = {'redash_host': self.redash_host, 'query_result_id': query_result_id}
        results_url = REDASH_QUERY_RESULTS_ENDPOINT.format(**url_inputs)
        resp = r.get(results_url, headers=self.headers)
        return resp.json()

    def get_preview_data(self, params: Dict, optionalHeaders: Dict = None) -> FlaskResponse:
        """
        Returns a FlaskResponse object, where the response data represents a json object
        with the preview data accessible on 'preview_data' key. The preview data should
        match amundsen_application.models.preview_data.PreviewDataSchema
        """
        LOGGER.debug('Retrieving preview data from Redash with params: %s', params)
        try:
            query_id = self.get_redash_query_id(params)
            if query_id is None:
                raise RedashQueryTemplateDoesNotExistForResource('Could not find query for params: %s', params)

            # Build headers to use the Query API key or User API key
            self._build_headers(params)

            query_params = self.build_redash_query_params(params)
            query_results, cached_result = self._start_redash_query(query_id=query_id, query_params=query_params)

            # Redash attempts to use internal caching. The format of the response
            # changes based on whether or not a cached response is returned
            if not cached_result:
                query_result_id = self._wait_for_query_finish(job_id=query_results['job']['id'])
                query_results = self._get_query_results(query_result_id=query_result_id)

            columns = [ColumnItem(c['name'], c['type']) for c in query_results['query_result']['data']['columns']]
            preview_data = PreviewData(columns, query_results['query_result']['data']['rows'])

            data = PreviewDataSchema().dump(preview_data)
            PreviewDataSchema().load(data)  # for validation only
            payload = jsonify({'preview_data': data})
            return make_response(payload, HTTPStatus.OK)

        except Exception as e:
            LOGGER.error('ERROR getting Redash preview: %s', e)
            return make_response(jsonify({'preview_data': {}}), HTTPStatus.INTERNAL_SERVER_ERROR)

    def get_feature_preview_data(self, params: Dict, optionalHeaders: Dict = None) -> FlaskResponse:
        pass



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import logging
import os

from typing import Dict, Optional

from amundsen_application.base.base_redash_preview_client import BaseRedashPreviewClient


LOGGER = logging.getLogger(__name__)


# Redash natively runs on port 5000, the same port as Amundsen.
# Make sure to update the running port to match your deployment!
DEFAULT_URL = 'http://localhost:5010'


# Update this mapping with your database.cluster and Redash query ID
SOURCE_DB_QUERY_MAP = {
    'snowflake.ca_covid': 1
}

# This example uses a common, system user, for the API key
REDASH_USER_API_KEY = os.environ.get('REDASH_USER_API_KEY', '')


def _build_db_cluster_key(params: Dict) -> str:
    _db = params.get('database')
    _cluster = params.get('cluster')

    db_cluster_key = f'{_db}.{_cluster}'
    return db_cluster_key


class RedashSimplePreviewClient(BaseRedashPreviewClient):
    def __init__(self,
                 *,
                 redash_host: str = DEFAULT_URL,
                 user_api_key: Optional[str] = REDASH_USER_API_KEY) -> None:
        super().__init__(redash_host=redash_host, user_api_key=user_api_key)

    def get_redash_query_id(self, params: Dict) -> Optional[int]:
        """
        Retrieves the query template that should be executed for the given
        source / database / schema / table combination.

        Redash Connections are generally unique to the source and database.
        For example, Snowflake account that has two databases would require two
        separate connections in Redash. This would require at least one query
        template per connection.

        The query ID can be found in the URL of the query when using the Redash GUI.
        """
        db_cluster_key = _build_db_cluster_key(params)
        return SOURCE_DB_QUERY_MAP.get(db_cluster_key)


class RedashComplexPreviewClient(BaseRedashPreviewClient):
    def __init__(self,
                 *,
                 redash_host: str = DEFAULT_URL,
                 user_api_key: Optional[str] = REDASH_USER_API_KEY) -> None:
        super().__init__(redash_host=redash_host, user_api_key=user_api_key)
        self.default_query_limit = 100
        self.max_redash_cache_age = 3600  # One Hour

    def _get_query_api_key(self, params: Dict) -> Optional[str]:
        if params.get('database') in ['redshift']:
            return os.environ.get('REDSHIFT_USER_API_KEY', '')
        return None

    def get_redash_query_id(self, params: Dict) -> Optional[int]:
        db_cluster_key = _build_db_cluster_key(params)
        return SOURCE_DB_QUERY_MAP.get(db_cluster_key)

    def get_select_fields(self, params: Dict) -> str:
        """
        Manually defining the dictionary in this function for readability
        """
        # These are sample values to show how table-level select clauses work
        field_select_vals = {
            'snowflake.ca_covid': {
                'open_data.case_demographics_age': (
                    "date, SUBSTR(age_group, 0, 2) || '******' as age_group, totalpositive, case_percent, ca_percent"
                ),
                'open_data.statewide_testing': 'date, tested'
            }
        }

        db_cluster_key = _build_db_cluster_key(params)
        schema_tbl_key = f"{params.get('schema')}.{params.get('tableName')}"

        # Always returns a value, defaults to '*' if nothing is defined
        return field_select_vals.get(db_cluster_key, {}).get(schema_tbl_key, '*')

    def get_where_clause(self, params: Dict) -> str:
        """
        MUST return the entire where clause, including the word "where"
        """
        where_vals = {
            'snowflake.ca_covid': {
                'open_data.case_demographics_age': "totalpositive < 120",
            }
        }

        db_cluster_key = _build_db_cluster_key(params)
        schema_tbl_key = f"{params.get('schema')}.{params.get('tableName')}"

        # Always returns a value, defaults to an empty string ('') if nothing is defined
        where_clause = where_vals.get(db_cluster_key, {}).get(schema_tbl_key, '')

        # Add the word where if a custom where clause is applied
        if where_clause:
            where_clause = f'WHERE {where_clause}'

        return where_clause



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import logging
from typing import Dict, List  # noqa: F401

from http import HTTPStatus

from flask import current_app as app

from amundsen_application.api.utils.request_utils import request_search

from amundsen_common.models.search import Filter, SearchRequest

from amundsen_application.models.user import dump_user, load_user

LOGGER = logging.getLogger(__name__)

# These can move to a configuration when we have custom use cases outside of these default values
valid_search_fields = {
    'table': {
        'badges',
        'column',
        'database',
        'schema',
        'table',
        'tag'
    },
    'dashboard': {
        'group_name',
        'name',
        'product',
        'tag'
    },
    'feature': {
        'badges',
        'entity',
        'feature_name',
        'feature_group',
        'tags'
    }
}


def map_dashboard_result(result: Dict) -> Dict:
    return {
        'type': 'dashboard',
        'key': result.get('key', None),
        'uri': result.get('uri', None),
        'url': result.get('url', None),
        'group_name': result.get('group_name', None),
        'name': result.get('name', None),
        'product': result.get('product', None),
        'tag': result.get('tag', None),
        'description': result.get('description', None),
        'last_successful_run_timestamp': result.get('last_successful_run_timestamp', None),
    }


def map_table_result(result: Dict) -> Dict:
    name = result.get('name') if result.get('name') else result.get('table')
    return {
        'type': 'table',
        'key': result.get('key', None),
        'name': name,
        'cluster': result.get('cluster', None),
        'description': result.get('description', None),
        'database': result.get('database', None),
        'schema': result.get('schema', None),
        'schema_description': result.get('schema_description', None),
        'badges': result.get('badges', None),
        'last_updated_timestamp': result.get('last_updated_timestamp', None),
    }


def map_feature_result(result: Dict) -> Dict:
    return {
        'type': 'feature',
        'description': result.get('description', None),
        'key': result.get('key', None),
        'last_updated_timestamp': result.get('last_updated_timestamp', None),
        'name': result.get('feature_name', None),
        'feature_group': result.get('feature_group', None),
        'version': result.get('version', None),
        'availability': result.get('availability', None),
        'entity': result.get('entity', None),
        'badges': result.get('badges', None),
        'status': result.get('status', None),
    }


def map_user_result(result: Dict) -> Dict:
    user_result = dump_user(load_user(result))
    user_result['type'] = 'user'
    return user_result


def generate_query_json(*, filters: Dict = {}, page_index: int, search_term: str) -> Dict:
    """
    Transforms the given paramaters to the query json for the search service according to
    the api defined at:
    https://github.com/lyft/amundsensearchlibrary/blob/master/search_service/api/swagger_doc/table/search_table_filter.yml
    https://github.com/lyft/amundsensearchlibrary/blob/master/search_service/api/swagger_doc/dashboard/search_dashboard_filter.yml
    """

    return {
        'page_index': int(page_index),
        'search_request': {
            'type': 'AND',
            'filters': filters
        },
        'query_term': search_term
    }


def execute_search_document_request(request_json: str, method: str) -> int:
    search_service_base = app.config['SEARCHSERVICE_BASE']
    search_document_url = f'{search_service_base}/v2/document'
    update_response = request_search(
        url=search_document_url,
        method=method,
        headers={'Content-Type': 'application/json'},
        data=request_json,
    )
    status_code = update_response.status_code
    if status_code != HTTPStatus.OK:
        LOGGER.info(f'Failed to execute {method} for {request_json} in searchservice, status code: {status_code}')
        LOGGER.info(update_response.text)

    return status_code


def generate_query_request(*, filters: List[Filter] = [],
                           resources: List[str] = [],
                           page_index: int = 0,
                           results_per_page: int = 10,
                           search_term: str) -> SearchRequest:

    return SearchRequest(query_term=search_term,
                         resource_types=resources,
                         page_index=page_index,
                         results_per_page=results_per_page,
                         filters=filters,
                         highlight_options={})



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import json
import logging

from http import HTTPStatus
from typing import cast

from flask import Response, jsonify, make_response, request, current_app as app
from flask.blueprints import Blueprint
from marshmallow import ValidationError
from werkzeug.utils import import_string

from amundsen_application.api.utils.request_utils import get_query_param
from amundsen_application.base.base_quality_client import BaseQualityClient

LOGGER = logging.getLogger(__name__)
QUALITY_CLIENT_INSTANCE = None

quality_blueprint = Blueprint('quality', __name__, url_prefix='/api/quality/v0')


def get_quality_client() -> BaseQualityClient:
    global QUALITY_CLIENT_INSTANCE
    if QUALITY_CLIENT_INSTANCE is None and app.config['QUALITY_CLIENT'] is not None:
        quality_client_class = import_string(app.config['QUALITY_CLIENT'])
        QUALITY_CLIENT_INSTANCE = quality_client_class()
    return cast(BaseQualityClient, QUALITY_CLIENT_INSTANCE)


@quality_blueprint.route('/table/summary', methods=['GET'])
def get_table_quality_checks_summary() -> Response:
    global QUALITY_CLIENT_INSTANCE
    try:
        client = get_quality_client()
        if client is not None:
            return _get_dq_checks_summary_client()
        payload = jsonify({'checks': {}, 'msg': 'This feature is not implemented'})
        return make_response(payload, HTTPStatus.NOT_IMPLEMENTED)
    except Exception as e:
        message = 'Encountered exception: ' + str(e)
        logging.exception(message)
        payload = jsonify({'checks': {}, 'msg': message})
        return make_response(payload, HTTPStatus.INTERNAL_SERVER_ERROR)


def _get_dq_checks_summary_client() -> Response:
    client = get_quality_client()
    table_key = get_query_param(request.args, 'key')
    response = client.get_table_quality_checks_summary(table_key=table_key)
    status_code = response.status_code
    if status_code == HTTPStatus.OK:
        try:
            quality_checks = json.loads(response.data).get('checks')
            payload = jsonify({'checks': quality_checks, 'msg': 'Success'})
        except ValidationError as err:
            logging.error('Quality data dump returned errors: ' + str(err.messages))
            raise Exception('The preview client did not return a valid Quality Checks object')
    else:
        message = 'Encountered error: Quality client request failed with code ' + str(status_code)
        logging.error(message)
        # only necessary to pass the error text
        payload = jsonify({'checks': {}, 'msg': message})
    return make_response(payload, status_code)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from amundsen_application.base.base_bigquery_preview_client import BaseBigqueryPreviewClient
from amundsen_application.models.preview_data import (
    PreviewData,
)
from google.cloud import bigquery
from flatten_dict import flatten


class BigqueryPreviewClient(BaseBigqueryPreviewClient):
    """
    Returns a Response object, where the response data represents a json object
    with the preview data accessible on 'preview_data' key. The preview data should
    match amundsen_application.models.preview_data.PreviewDataSchema
    """

    def __init__(self) -> None:
        # Requires access to a service account eg.
        # GOOGLE_APPLICATION_CREDENTIALS=path/serviceaccount.json or a mounted service kubernetes service account.
        super().__init__(bq_client=bigquery.Client("your project here"))

    def _bq_list_rows(
        self, gcp_project_id: str, table_project_name: str, table_name: str
    ) -> PreviewData:
        """
        Returns PreviewData from bigquery list rows api.
        """
        table_id = f"{gcp_project_id}.{table_project_name}.{table_name}"
        rows = self.bq_client.list_rows(table_id, max_results=self.preview_limit)

        # Make flat key ColumnItems from table schema.
        columns = []
        for field in rows.schema:
            extend_with = self._column_item_from_bq_schema(field)
            columns.extend(extend_with)

        # Flatten rows and set missing empty keys to None, to avoid errors with undefined values
        # in frontend
        column_data = []
        for row in rows:
            flat_row = flatten(dict(row), reducer="dot")
            for key in columns:
                if key.column_name not in flat_row:
                    flat_row[key.column_name] = None
            column_data.append(flat_row)

        return PreviewData(columns, column_data)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import abc
from typing import Any

from amundsen_application.models.data_issue import DataIssue
from amundsen_application.models.issue_results import IssueResults


class BaseIssueTrackerClient(abc.ABC):
    @abc.abstractmethod
    def __init__(self) -> None:
        pass  # pragma: no cover

    @abc.abstractmethod
    def get_issues(self, table_uri: str) -> IssueResults:
        """
        Gets issues from the issue tracker
        :param table_uri: Table Uri ie databasetype://database/table
        :return:
        """
        raise NotImplementedError  # pragma: no cover

    @abc.abstractmethod
    def create_issue(self,
                     table_uri: str,
                     title: str,
                     description: str,
                     priority_level: str,
                     table_url: str,
                     **kwargs: Any) -> DataIssue:
        """
        Given a title, description, and table key, creates a ticket in the configured project
        Automatically places the table_uri in the description of the ticket.
        Returns the ticket information, including URL.
        :param description: User provided description for the jira ticket
        :param priority_level: Priority level for the ticket
        :param table_uri: Table URI ie databasetype://database/table
        :param title: Title of the ticket
        :param table_url: Link to access the table
        :return: A single ticket
        """
        raise NotImplementedError  # pragma: no cover



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from http import HTTPStatus
import logging
from typing import Any, Dict, List
from amundsen_application.base.base_preview_client import BasePreviewClient
from amundsen_application.models.preview_data import (
    ColumnItem,
    PreviewData,
    PreviewDataSchema,
)
from flask import Response, make_response, jsonify
from marshmallow import ValidationError
from google.cloud import bigquery

import json
import decimal


class BaseBigqueryPreviewClient(BasePreviewClient):
    """
    Returns a Response object, where the response data represents a json object
    with the preview data accessible on 'preview_data' key. The preview data should
    match amundsen_application.models.preview_data.PreviewDataSchema
    """

    def __init__(self, bq_client: bigquery.Client, preview_limit: int = 5, previewable_projects: List = None) -> None:
        # Client passed from custom implementation. See example implementation.
        self.bq_client = bq_client
        self.preview_limit = preview_limit
        # List of projects that are approved for whitelisting. None(Default) approves all google projects.
        self.previewable_projects = previewable_projects

    def _bq_list_rows(
        self, gcp_project_id: str, table_project_name: str, table_name: str
    ) -> PreviewData:
        """
        Returns PreviewData from bigquery list rows api.
        """
        pass  # pragma: no cover

    def _column_item_from_bq_schema(self, schemafield: bigquery.SchemaField, key: str = None) -> List:
        """
        Recursively build ColumnItems from the bigquery schema
        """
        all_fields = []
        if schemafield.field_type != "RECORD":
            name = key + "." + schemafield.name if key else schemafield.name
            return [ColumnItem(name, schemafield.field_type)]
        for field in schemafield.fields:
            if key:
                name = key + "." + schemafield.name
            else:
                name = schemafield.name
            all_fields.extend(self._column_item_from_bq_schema(field, name))
        return all_fields

    def get_preview_data(self, params: Dict, optionalHeaders: Dict = None) -> Response:
        if self.previewable_projects and params["cluster"] not in self.previewable_projects:
            return make_response(jsonify({"preview_data": {}}), HTTPStatus.FORBIDDEN)

        preview_data = self._bq_list_rows(
            params["cluster"],
            params["schema"],
            params["tableName"],
        )
        try:
            data = PreviewDataSchema().dump(preview_data)
            PreviewDataSchema().load(data)  # for validation only
            payload = json.dumps({"preview_data": data}, cls=Encoder)
            return make_response(payload, HTTPStatus.OK)
        except ValidationError as err:
            logging.error("PreviewDataSchema serialization error + " + str(err.messages))
            return make_response(
                jsonify({"preview_data": {}}), HTTPStatus.INTERNAL_SERVER_ERROR
            )

    def get_feature_preview_data(self, params: Dict, optionalHeaders: Dict = None) -> Response:
        pass


class Encoder(json.JSONEncoder):
    """
    Customized json encoder class to address the parsing of decimal/numeric data types into float.
    """

    def default(self, obj: Any) -> Any:
        if isinstance(obj, decimal.Decimal):
            return float(obj)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import abc
from typing import Dict, List, Optional

from flask import Response


class BaseMailClient(abc.ABC):
    @abc.abstractmethod
    def __init__(self, recipients: List[str]) -> None:
        pass  # pragma: no cover

    @abc.abstractmethod
    def send_email(self,
                   html: str,
                   subject: str,
                   optional_data: Optional[Dict] = None,
                   recipients: Optional[List[str]] = None,
                   sender: Optional[str] = None) -> Response:
        """
        Sends an email using the following parameters
        :param html: HTML email content
        :param subject: The subject of the email
        :param optional_data: An optional dictionary of any values needed for custom implementations
        :param recipients: An optional list of recipients for the email, the implementation
            for this class should determine whether to use the recipients from the function,
            the __init__ or both
        :param sender: An optional sending address associated with the email, the implementation
            should determine whether to use this value or another (e.g. from envvars)
        :return:
        """
        raise NotImplementedError  # pragma: no cover



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from abc import ABCMeta, abstractmethod


class BasePreview(metaclass=ABCMeta):
    """
    A Preview interface for other product to implement. For example, see ModePreview.
    """

    @abstractmethod
    def get_preview_image(self, *, uri: str) -> bytes:
        """
        Returns image bytes given URI
        :param uri:
        :return:
        :raises: FileNotFound when either Report is not available or Preview image is not available
        """
        pass



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import os
from amundsen_application import create_app


if not os.getenv('FRONTEND_SVC_CONFIG_MODULE_CLASS'):
    os.environ['FRONTEND_SVC_CONFIG_MODULE_CLASS'] = 'amundsen_application.config.TestConfig'

application = create_app()

if __name__ == '__main__':
    application.run(host='0.0.0.0')



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from flask import current_app as app
from threading import Lock
from werkzeug.utils import import_string

from amundsen_application.base.base_issue_tracker_client import BaseIssueTrackerClient

_issue_tracker_client = None
_issue_tracker_client_lock = Lock()


def get_issue_tracker_client() -> BaseIssueTrackerClient:
    """
    Provides singleton proxy client based on the config
    :return: Proxy instance of any subclass of BaseProxy
    """
    global _issue_tracker_client

    if _issue_tracker_client:
        return _issue_tracker_client

    with _issue_tracker_client_lock:
        if _issue_tracker_client:
            return _issue_tracker_client
        else:
            # Gather all the configuration to create an IssueTrackerClient
            if app.config['ISSUE_TRACKER_CLIENT_ENABLED']:
                url = app.config['ISSUE_TRACKER_URL']
                user = app.config['ISSUE_TRACKER_USER']
                password = app.config['ISSUE_TRACKER_PASSWORD']
                project_id = app.config['ISSUE_TRACKER_PROJECT_ID']
                max_results = app.config['ISSUE_TRACKER_MAX_RESULTS']
                issue_labels = app.config['ISSUE_LABELS']

                if app.config['ISSUE_TRACKER_CLIENT']:
                    client = import_string(app.config['ISSUE_TRACKER_CLIENT'])
                    _issue_tracker_client = client(issue_labels=issue_labels,
                                                   issue_tracker_url=url,
                                                   issue_tracker_user=user,
                                                   issue_tracker_password=password,
                                                   issue_tracker_project_id=project_id,
                                                   issue_tracker_max_results=max_results)

    return _issue_tracker_client



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import abc
import logging
from http import HTTPStatus
from typing import Dict

from amundsen_application.base.base_preview_client import BasePreviewClient
from amundsen_application.models.preview_data import (PreviewData,
                                                      PreviewDataSchema)
from flask import Response as FlaskResponse
from flask import jsonify, make_response
from marshmallow import ValidationError


class BaseS3PreviewClient(BasePreviewClient):
    def __init__(self) -> None:
        pass

    @abc.abstractmethod
    def get_s3_preview_data(self, *, params: Dict) -> PreviewData:
        """
        Returns the data from S3 in PreviewData model format
        """
        pass  # pragma: no cover

    def get_preview_data(self, params: Dict, optionalHeaders: Dict = None) -> FlaskResponse:
        try:
            preview_data = self.get_s3_preview_data(params=params)
            try:
                data = PreviewDataSchema().dump(preview_data)
                PreviewDataSchema().load(data)  # for validation only
                payload = jsonify({'preview_data': data})
                return make_response(payload, HTTPStatus.OK)
            except ValidationError as err:
                logging.error("PreviewDataSchema serialization error " + str(err.messages))
                return make_response(jsonify({'preview_data': {}}), HTTPStatus.INTERNAL_SERVER_ERROR)
        except Exception as err:
            logging.error("error getting s3 preview data " + str(err))
            return make_response(jsonify({'preview_data': {}}), HTTPStatus.INTERNAL_SERVER_ERROR)

    def get_feature_preview_data(self, params: Dict, optionalHeaders: Dict = None) -> FlaskResponse:
        """
        BaseS3PreviewClient only supports data preview currently but this function needs to be stubbed to
        implement the BasePreviewClient interface
        """
        pass



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import json
import logging
from http import HTTPStatus
from typing import Any, Dict, List  # noqa: F401

from amundsen_common.models.search import (Filter, SearchRequestSchema,
                                           SearchResponseSchema)
from flask import Response
from flask import current_app as app
from flask import jsonify, make_response, request
from flask.blueprints import Blueprint

from amundsen_application.log.action_log import action_logging
from amundsen_application.api.utils.request_utils import (get_query_param,
                                                          request_search)
from amundsen_application.api.utils.search_utils import (
    generate_query_request, map_dashboard_result, map_feature_result,
    map_table_result, map_user_result)

LOGGER = logging.getLogger(__name__)

REQUEST_SESSION_TIMEOUT_SEC = 3

SEARCH_ENDPOINT = '/v2/search'

RESOURCE_TO_MAPPING = {
    'table': map_table_result,
    'dashboard': map_dashboard_result,
    'feature': map_feature_result,
    'user': map_user_result,
}

DEFAULT_FILTER_OPERATION = 'OR'

search_blueprint = Blueprint('search', __name__, url_prefix='/api/search/v1')


def _transform_filters(filters: Dict, resources: List[str]) -> List[Filter]:
    transformed_filters = []
    searched_resources_with_filters = set(filters.keys()).intersection(resources)
    for resource in searched_resources_with_filters:
        resource_filters = filters[resource]
        for field in resource_filters.keys():
            field_filters = resource_filters[field]
            values = []
            filter_operation = DEFAULT_FILTER_OPERATION

            if field_filters is not None and field_filters.get('value') is not None:
                value_str = field_filters.get('value')
                values = [str.strip() for str in value_str.split(',') if str != '']
                filter_operation = field_filters.get('filterOperation', DEFAULT_FILTER_OPERATION)

            transformed_filters.append(Filter(name=field,
                                              values=values,
                                              operation=filter_operation))

    return transformed_filters


@search_blueprint.route('/search', methods=['POST'])
def search() -> Response:
    """
    Parse the request arguments and call the helper method to execute a search for specified resources
    :return: a Response created with the results from the helper method
    """
    results_dict = {}
    try:
        request_json = request.get_json()
        search_term = get_query_param(request_json, 'searchTerm', '"searchTerm" parameter expected in request data')
        page_index = get_query_param(request_json, 'pageIndex', '"pageIndex" parameter expected in request data')
        results_per_page = get_query_param(request_json,
                                           'resultsPerPage',
                                           '"resultsPerPage" parameter expected in request data')
        search_type = request_json.get('searchType')
        resources = request_json.get('resources', [])
        filters = request_json.get('filters', {})
        results_dict = _search_resources(search_term=search_term,
                                         resources=resources,
                                         page_index=int(page_index),
                                         results_per_page=int(results_per_page),
                                         filters=filters,
                                         search_type=search_type)
        return make_response(jsonify(results_dict), results_dict.get('status_code', HTTPStatus.OK))
    except Exception as e:
        message = 'Encountered exception: ' + str(e)
        LOGGER.exception(message)
        return make_response(jsonify(results_dict), HTTPStatus.INTERNAL_SERVER_ERROR)


@action_logging
def _search_resources(*, search_term: str,
                      resources: List[str],
                      page_index: int,
                      results_per_page: int,
                      filters: Dict,
                      search_type: str) -> Dict[str, Any]:
    """
    Call the search service endpoint and return matching results
    :return: a json output containing search results array as 'results'
    """
    default_results = {
        'page_index': int(page_index),
        'results': [],
        'total_results': 0,
    }

    results_dict = {
        'search_term': search_term,
        'msg': '',
        'table': default_results,
        'dashboard': default_results,
        'feature': default_results,
        'user': default_results,
    }

    try:
        transformed_filters = _transform_filters(filters=filters, resources=resources)
        query_request = generate_query_request(filters=transformed_filters,
                                               resources=resources,
                                               page_index=page_index,
                                               results_per_page=results_per_page,
                                               search_term=search_term)
        request_json = json.dumps(SearchRequestSchema().dump(query_request))
        url_base = app.config['SEARCHSERVICE_BASE'] + SEARCH_ENDPOINT
        response = request_search(url=url_base,
                                  headers={'Content-Type': 'application/json'},
                                  method='POST',
                                  data=request_json)
        status_code = response.status_code

        if status_code == HTTPStatus.OK:
            search_response = SearchResponseSchema().loads(json.dumps(response.json()))
            results_dict['msg'] = search_response.msg
            results = search_response.results
            for resource in results.keys():
                results_dict[resource] = {
                    'page_index': int(page_index),
                    'results': [RESOURCE_TO_MAPPING[resource](result) for result in results[resource]['results']],
                    'total_results': results[resource]['total_results'],
                }
        else:
            message = 'Encountered error: Search request failed'
            results_dict['msg'] = message

        results_dict['status_code'] = status_code
        return results_dict

    except Exception as e:
        message = f'Encountered exception: {str(e)}'
        results_dict['msg'] = message
        LOGGER.exception(message)
        return results_dict



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from abc import ABCMeta, abstractmethod
from flask import Response


class BaseQualityClient(metaclass=ABCMeta):
    """
    An abstract interface for a Quality Service Client
    """

    @abstractmethod
    def get_table_quality_checks_summary(self, *, table_key: str) -> Response:
        """
        Returns table quality checks for a given table uri
        :param table_key: Table key for the table whose table quality
        :return: TableQualityChecks object
        """
        raise NotImplementedError  # pragma: no cover

    @abstractmethod
    def get_table_quality_checks(self, *, table_key: str) -> bytes:
        """
        Returns table quality checks for a given table uri
        :param table_key: Table key for the table whose table quality
        :return: TableQualityChecks object
        """
        raise NotImplementedError  # pragma: no cover



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import logging
from pkg_resources import iter_entry_points

from http import HTTPStatus

from flask import Response, jsonify, make_response, current_app as app
from flask.blueprints import Blueprint
from werkzeug.utils import import_string

LOGGER = logging.getLogger(__name__)
ANNOUNCEMENT_CLIENT_CLASS = None
ANNOUNCEMENT_CLIENT_INSTANCE = None

for entry_point in iter_entry_points(group='announcement_client', name='announcement_client_class'):
    announcement_client_class = entry_point.load()
    if announcement_client_class is not None:
        ANNOUNCEMENT_CLIENT_CLASS = announcement_client_class

announcements_blueprint = Blueprint('announcements', __name__, url_prefix='/api/announcements/v0')


@announcements_blueprint.route('/', methods=['GET'])
def get_announcements() -> Response:
    global ANNOUNCEMENT_CLIENT_INSTANCE
    global ANNOUNCEMENT_CLIENT_CLASS
    try:
        if ANNOUNCEMENT_CLIENT_INSTANCE is None:
            if ANNOUNCEMENT_CLIENT_CLASS is not None:
                ANNOUNCEMENT_CLIENT_INSTANCE = ANNOUNCEMENT_CLIENT_CLASS()
                logging.warn('Setting announcement_client via entry_point is DEPRECATED'
                             ' and will be removed in a future version')
            elif (app.config['ANNOUNCEMENT_CLIENT_ENABLED']
                    and app.config['ANNOUNCEMENT_CLIENT'] is not None):
                ANNOUNCEMENT_CLIENT_CLASS = import_string(app.config['ANNOUNCEMENT_CLIENT'])
                ANNOUNCEMENT_CLIENT_INSTANCE = ANNOUNCEMENT_CLIENT_CLASS()
            else:
                payload = jsonify({'posts': [],
                                   'msg': 'A client for retrieving announcements must be configured'})
                return make_response(payload, HTTPStatus.NOT_IMPLEMENTED)
        return ANNOUNCEMENT_CLIENT_INSTANCE._get_posts()
    except Exception as e:
        message = 'Encountered exception: ' + str(e)
        logging.exception(message)
        payload = jsonify({'posts': [], 'msg': message})
        return make_response(payload, HTTPStatus.INTERNAL_SERVER_ERROR)
